@@ -14,7 +14,7 @@
 -- >
 -- > f :: Text -> Text -> Text
 -- > f a b =
--- >   [text|
+-- >   [trimming|
 -- >     function(){
 -- >       function(){
 -- >         $a
@@ -25,7 +25,7 @@
 --
 -- Executing the following:
 --
--- > main = T.putStrLn $ f "1" "2"
+-- > main = Text.putStrLn $ f "1" "2"
 --
 -- will produce this (notice the reduced indentation compared to how it was
 -- declared):
@@ -39,7 +39,7 @@
 --
 -- Now let's test it with multiline string parameters:
 --
--- > main = T.putStrLn $ f
+-- > main = Text.putStrLn $ f
 -- >   "{\n  indented line\n  indented line\n}"
 -- >   "{\n  indented line\n  indented line\n}"
 --
@@ -64,7 +64,7 @@
 -- If you need to separate variable placeholder from the following text to
 -- prevent treating the rest of line as variable name, use escaped variable:
 --
--- > f name = [text|this_could_be_${name}_long_identifier|]
+-- > f name = [trimming|this_could_be_${name}_long_identifier|]
 --
 -- So
 --
@@ -73,55 +73,69 @@
 -- If you want to write something that looks like a variable but should be
 -- inserted as-is, escape it with another @$@:
 --
--- > f word = [text|$$my ${word} $${string}|]
+-- > f word = [trimming|$$my ${word} $${string}|]
 --
 -- results in
 --
--- > f "funny" == "$my funny ${string}|]
-module NeatInterpolation (text) where
+-- > f "funny" == "$my funny ${string}"
+module NeatInterpolation (trimming, untrimming, text) where
 
 import NeatInterpolation.Prelude
-
 import Language.Haskell.TH
 import Language.Haskell.TH.Quote hiding (quoteExp)
-
-import NeatInterpolation.String
-import NeatInterpolation.Parsing
-
-import Data.Text (Text)
-import qualified Data.Text as T
+import qualified Data.Text as Text
+import qualified NeatInterpolation.String as String
+import qualified NeatInterpolation.Parsing as Parsing
 
 
--- |
--- The quasiquoter.
-text :: QuasiQuoter
-text = QuasiQuoter quoteExp notSupported notSupported notSupported where
+expQQ quoteExp = QuasiQuoter quoteExp notSupported notSupported notSupported where
   notSupported _ = fail "Quotation in this context is not supported"
 
+{-|
+An alias to `trimming` for backward-compatibility.
+-}
+text :: QuasiQuoter
+text = trimming
+
+{-|
+Trimmed quasiquoter variation.
+Same as `untrimming`, but also
+removes the leading and trailing whitespace.
+-}
+trimming :: QuasiQuoter
+trimming = expQQ (quoteExp . String.trim . String.unindent . String.tabsToSpaces)
+
+{-|
+Untrimmed quasiquoter variation.
+Unindents the quoted template and converts tabs to spaces.
+-}
+untrimming :: QuasiQuoter
+untrimming = expQQ (quoteExp . String.unindent . String.tabsToSpaces)
+
 indentQQPlaceholder :: Int -> Text -> Text
-indentQQPlaceholder indent text = case T.lines text of
-  head:tail -> T.intercalate (T.singleton '\n') $
-               head : map (T.replicate indent (T.singleton ' ') <>) tail
+indentQQPlaceholder indent text = case Text.lines text of
+  head:tail -> Text.intercalate (Text.singleton '\n') $
+               head : map (Text.replicate indent (Text.singleton ' ') <>) tail
   [] -> text
 
 quoteExp :: String -> Q Exp
 quoteExp input =
-  case parseLines $ normalizeQQInput input of
+  case Parsing.parseLines input of
     Left e -> fail $ show e
-    Right lines -> sigE (appE [|T.unlines|] $ listE $ map lineExp lines)
+    Right lines -> sigE (appE [|Text.intercalate (Text.singleton '\n')|] $ listE $ map lineExp lines)
                         [t|Text|]
 
-lineExp :: Line -> Q Exp
-lineExp (Line indent contents) =
+lineExp :: Parsing.Line -> Q Exp
+lineExp (Parsing.Line indent contents) =
   case contents of
-    []  -> [| T.empty |]
+    []  -> [| Text.empty |]
     [x] -> toExp x
-    xs  -> appE [|T.concat|] $ listE $ map toExp xs
+    xs  -> appE [|Text.concat|] $ listE $ map toExp xs
   where toExp = contentExp (fromIntegral indent)
 
-contentExp :: Integer -> LineContent -> Q Exp
-contentExp _ (LineContentText text) = appE [|T.pack|] (stringE text)
-contentExp indent (LineContentIdentifier name) = do
+contentExp :: Integer -> Parsing.LineContent -> Q Exp
+contentExp _ (Parsing.LineContentText text) = appE [|Text.pack|] (stringE text)
+contentExp indent (Parsing.LineContentIdentifier name) = do
   valueName <- lookupValueName name
   case valueName of
     Just valueName -> do
